## ddpcRquant Shiny Application: user interface ##

library(shiny)

shinyUI(  
  pageWithSidebar(
  headerPanel("ddpcRquant"),
  
  sidebarPanel(
    conditionalPanel(condition="input.conditionedPanels==1",
                     
                     helpText("Input Data"), 
                     tags$hr(),
                     
                     selectInput('assay_demo', 
                     'Choose Input Data',
                     choices = c('Upload ddpcrfiles', 'Demo Dataset'),
                     selected="Upload ddpcrfiles"),
                     
                     fileInput('files', 
                               'Upload CSV Files',
                               accept=c('text/csv', 
                                        'text/comma-separated-values,text/plain', 
                                        '.csv'),
                               multiple=TRUE),
                     tags$hr(),
                     selectInput('qx', 'Choose BioRad QX system',
                                 choices= c('qx100', 'qx200'),
                                 selected="qx200"),
                     tags$hr(),
                     actionButton("start", label="Start Analysis"),
                     useShinyjs(),
                     extendShinyjs(text = jsResetCode), 
                     actionButton("reset", label="Reset Analysis")
                     #,tags$hr()
    ),
    conditionalPanel(condition="input.conditionedPanels==2",
                     helpText("Overview detected Assays")#,
                     #selectInput('assay_choice', 
                     #            'Choose Assay',
                     #            choices = c('Assay1'))
                     #choices = assays)
    ),
    
    conditionalPanel(condition="input.conditionedPanels==3",
                     
      uiOutput('assays'),
      
      tags$hr(),
      
      numericInput("vol.mix", "Volume Mix (µL):", 20),
    
      numericInput("vol.tmp", "Volume DNA Template (µL):", 1),
      
      sliderInput("int.threshold", "Threshold Interval:",
                    min=99.90,
                    max=100,
                    value=99.95,
                    step=0.001),
 
      numericInput("man.threshold", "Manual Threshold:", 0),
      
      checkboxInput("advanced", label="Advanced Options"),
      
      conditionalPanel(
        condition = "input.advanced == true",
        
         sliderInput("reps",
                     "Number of repititions:",
                     min = 1,
                     max = 500,
                     value = 10),
        
         sliderInput("blocks",
                      "Number of blocks:",
                      min = 1,
                      max = 250,
                     value = 150),
        
         selectInput("CI_calculation", "CI calculation method:", 
                    choices = c("wilson","agresti-coull", "exact", "prop.test", 
                                "profile", "lrt", "asymptotic",
                                "bayes", "cloglog", "logit", "probit")),
        
         sliderInput("ci_conc", "CI concentration Interval:",
                    min=0.90,
                    max=1.00,
                    value=0.95,
                    step=0.01)           
        ),

      tags$hr(),
      actionButton("go", "Go")  

    ),
    conditionalPanel(condition="input.conditionedPanels==6",
                     h4("Replicate analysis"),
                     p("Start replicate analyis (this may take a while)"),
                     #p("For triplicate samples: 30sec/sample")
                     tags$hr(),
                     
                     actionButton("start_rep", "Start")        
    )
  ),
  
  mainPanel(
    tabsetPanel(
            
      tabPanel("Data Input", 
               value=1, 
               tableOutput("files")
               ), 
      
      tabPanel("Overview Assays", 
               value=2,
               h4("Detected Assays"),
               tableOutput("assay_info"),
               h4("NTCs per assay"),
               tableOutput("assay_ntcs"),
               h4("Samples per assay"),
               tableOutput("assay_samples")
               ), 
      
      tabPanel("NTC Threshold Analysis", 
               value=3,
               #progressbar
               #verbatimTextOutput("choice"),
               plotOutput("ntc_plot"),
               downloadButton('downloadPlot', 'Download')
      
      ), 
      
      tabPanel("Sample Analysis", 
               value=4,
               #plotOutput("sample_plots")
               #progressbar
               uiOutput("plots"),
               downloadButton('downloadPlots', 'Download')
      ), 

      tabPanel("Summary", 
               h4("Summary"),
               value=5,
               tableOutput("summary"),
               downloadButton('downloadData', 'Download'),
               h4("Summary Parameters"),
               tableOutput("summary_params"),
               p("Please refer to this information in the Material & Methods section of your publication"),
               downloadButton('downloadParam', 'Download')#, 
               #h4("Replicate analysis"),
               #tableOutput("replicate"),
               #downloadButton('downloadRepl', 'Download'),
               #h4("Replicate analysis Plot"),
               #plotOutput("globalplot"),
               #downloadButton('downloadglobal', 'Download')
            
      ),

      tabPanel("Replicate Analysis", 

      h4("Replicate analysis"),
      tableOutput("replicate"),
      downloadButton('downloadRepl', 'Download'),
      h4("Replicate analysis Plot"),
      plotOutput("globalplot"),
      downloadButton('downloadglobal', 'Download'),
      value=6
      ),
      
      tabPanel("How To Use", 
               h4("How To Use this ddpcrQuant application"),
               p("This application works with an analysis spread over consecutive tabs."),
               p("Please work your way through from left to right"),
               h4("1. Data Input"),
               p("a) Select whether you want to upload your own data or work on a preloaded demodataset"),
               p("b) If you want to upload your data, browse to your files and select the headfile and amplitudefiles"),
               p("c) Press Start"),
               p("d) Check whether all your desired files are uploaded"),
               h4("2. Overview assays"),
               p("Check whether all your desired assays are listed"),
               p("If this is not the case, make sure you have uploaded all you files or check your annotation in the headfile"),
               #p("Annotation requirements:"),
               h4("3. NTC Threshold Analysis"),
               p("Select the desired assay to be run, check parameters and press Go"),
               p("Evaluate threshold, adjust parameters if necessary and press Go"),
               h4("4. Sample Analysis"),
               p("This will go automatically upon clicking the tab. Please note that this step can take a while."),
               h4("5. Summary"),
               p("This will go automatically upon clicking the tab. Please note that this step can take a while."),
               value=7
      ),

      id = "conditionedPanels"
    )
    
  )
))

